/*
 *  wlu - Teletronics WL24xx Configuration Utility.
 *  Ported to FreeBSD 3.4 by WebWorld Warehouse Ltd.
 *  Contact: wireless@valemount.com
 *  Website: www.webworldwarehouse.com
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <termios.h>
#include <sys/socket.h>
#include <string.h>
#include <net/if.h>
#include "wlu.h"

/* */
/* define CCR in EXTRA MIB */
/* */
#define CCR_AP          0
#define CCR_INFRA       1
#define CCR_ADHOC       2
#define CCR_IAP         3

WORD CRC;

WORD CRCTAB[256] = {
	 0x0000,0x1021,0x2042,0x3063,0x4084,0x50a5,0x60c6,0x70e7,
	 0x8108,0x9129,0xa14a,0xb16b,0xc18c,0xd1ad,0xe1ce,0xf1ef,
	 0x1231,0x0210,0x3273,0x2252,0x52b5,0x4294,0x72f7,0x62d6,
	 0x9339,0x8318,0xb37b,0xa35a,0xd3bd,0xc39c,0xf3ff,0xe3de,
	 0x2462,0x3443,0x0420,0x1401,0x64e6,0x74c7,0x44a4,0x5485,
	 0xa56a,0xb54b,0x8528,0x9509,0xe5ee,0xf5cf,0xc5ac,0xd58d,
	 0x3653,0x2672,0x1611,0x0630,0x76d7,0x66f6,0x5695,0x46b4,
	 0xb75b,0xa77a,0x9719,0x8738,0xf7df,0xe7fe,0xd79d,0xc7bc,
	 0x48c4,0x58e5,0x6886,0x78a7,0x0840,0x1861,0x2802,0x3823,
	 0xc9cc,0xd9ed,0xe98e,0xf9af,0x8948,0x9969,0xa90a,0xb92b,
	 0x5af5,0x4ad4,0x7ab7,0x6a96,0x1a71,0x0a50,0x3a33,0x2a12,
	 0xdbfd,0xcbdc,0xfbbf,0xeb9e,0x9b79,0x8b58,0xbb3b,0xab1a,
	 0x6ca6,0x7c87,0x4ce4,0x5cc5,0x2c22,0x3c03,0x0c60,0x1c41,
	 0xedae,0xfd8f,0xcdec,0xddcd,0xad2a,0xbd0b,0x8d68,0x9d49,
	 0x7e97,0x6eb6,0x5ed5,0x4ef4,0x3e13,0x2e32,0x1e51,0x0e70,
	 0xff9f,0xefbe,0xdfdd,0xcffc,0xbf1b,0xaf3a,0x9f59,0x8f78,
	 0x9188,0x81a9,0xb1ca,0xa1eb,0xd10c,0xc12d,0xf14e,0xe16f,
	 0x1080,0x00a1,0x30c2,0x20e3,0x5004,0x4025,0x7046,0x6067,
	 0x83b9,0x9398,0xa3fb,0xb3da,0xc33d,0xd31c,0xe37f,0xf35e,
	 0x02b1,0x1290,0x22f3,0x32d2,0x4235,0x5214,0x6277,0x7256,
	 0xb5ea,0xa5cb,0x95a8,0x8589,0xf56e,0xe54f,0xd52c,0xc50d,
	 0x34e2,0x24c3,0x14a0,0x0481,0x7466,0x6447,0x5424,0x4405,
	 0xa7db,0xb7fa,0x8799,0x97b8,0xe75f,0xf77e,0xc71d,0xd73c,
	 0x26d3,0x36f2,0x0691,0x16b0,0x6657,0x7676,0x4615,0x5634,
	 0xd94c,0xc96d,0xf90e,0xe92f,0x99c8,0x89e9,0xb98a,0xa9ab,
	 0x5844,0x4865,0x7806,0x6827,0x18c0,0x08e1,0x3882,0x28a3,
	 0xcb7d,0xdb5c,0xeb3f,0xfb1e,0x8bf9,0x9bd8,0xabbb,0xbb9a,
	 0x4a75,0x5a54,0x6a37,0x7a16,0x0af1,0x1ad0,0x2ab3,0x3a92,
	 0xfd2e,0xed0f,0xdd6c,0xcd4d,0xbdaa,0xad8b,0x9de8,0x8dc9,
	 0x7c26,0x6c07,0x5c64,0x4c45,0x3ca2,0x2c83,0x1ce0,0x0cc1,
	 0xef1f,0xff3e,0xcf5d,0xdf7c,0xaf9b,0xbfba,0x8fd9,0x9ff8,
	 0x6e17,0x7e36,0x4e55,0x5e74,0x2e93,0x3eb2,0x0ed1,0x1ef0
};

#define XCRC(CRC,cp) (CRCTAB[((CRC>>8)&255)^cp]^(CRC<<8))

WORD CRC_BLOCK(BYTE *ptr, WORD Count)
{    
	WORD i;	
	for ( i=0; i<Count; i++, ptr++){ 
		  CRC=XCRC(CRC,(BYTE) *ptr);   
	}
	return CRC;
}
int             g_fd;
struct ifreq    g_ifreq;
WL_IOCTL_BLK    *g_pBlk;
unsigned char   g_userdata[1024];

typedef struct _WL_CMD {
    const char *cpszCmd;
    int         nArgc;
    BOOL        (*CallFunc)();
} WL_CMD;

BOOL WL_CmdReset(const char *cpszArgv[])
{
    if (strcasecmp(cpszArgv[0], "now") != 0) {
        printf("ERROR: RESET is write-only! Try 'reset now'!\n");
        return FALSE;
    }

    g_pBlk->uCmd = WL_IOCTL_SET_RESET;
    *g_pBlk->pData = 0;
    
    if (ioctl(g_fd, SIOC_WL_IOCTL, &g_ifreq) == -1) {
        perror("ERROR");
        return FALSE;
    }
    
    printf("RESET NOW\n");
    return TRUE;
}

BOOL WL_CmdFlash(const char *cpszArgv[])
{
	char filename[50];
	BYTE	buff[1028];
	FILE *fptr;
	DWORD ftop,fbottom,fkeep,flength,i,flashaddress;
	WORD chk_crc,crc16;
	flashaddress=0x4000;
	strcpy(filename,cpszArgv[0]);
	fptr=fopen(filename,"r+b");
	if (! fptr){
		printf("ERROR: Cannot open %s\n",filename);
		return FALSE;
	}
	fseek(fptr,0,SEEK_END);
	flength=ftell(fptr);
	fseek(fptr, flength-2,SEEK_SET);
	fread(&chk_crc,1,2,fptr);
	flength-=2;
	CRC=0;
	fseek(fptr,0,SEEK_SET);
	for (i=0;i<flength;i=i+1024){
		if (flength<i+1024){
			fread(&buff[4],sizeof(char),flength-i,fptr);
			crc16 = CRC_BLOCK((BYTE *) &buff[4],flength-i);
		}
		else{
			fread(&buff[4],sizeof(char),1024,fptr);
			crc16 = CRC_BLOCK((BYTE *) &buff[4],1024);
		}
	}
	if (crc16 != chk_crc){
		printf("CRC ERROR %x\n",crc16);
	        return FALSE;
	}
	fseek(fptr,0,SEEK_SET);
	for (i=0;i<flength;i=i+1024){
		if (flength<i+1024){
			fread(&buff[4],sizeof(char),flength-i,fptr);
			g_pBlk->uLen = flength-i+4;
		}
		else{
			fread(&buff[4],sizeof(char),1024,fptr);
			g_pBlk->uLen = 1024+4;
		}
		*(DWORD *) buff= flashaddress;
    		g_pBlk->uCmd = WL_IOCTL_WRITE_FLASH;
    		g_pBlk->pData = buff;
    		if (ioctl(g_fd, SIOC_WL_IOCTL, &g_ifreq) == -1) {
        		perror("ERROR");
			fclose(fptr);
			return FALSE;
		}
		printf(".");
		flashaddress+=(g_pBlk->uLen-4);
	}
	fclose(fptr);
	printf("\nFirmware upgrade success\n");
	return TRUE;
	
}


BOOL WL_CmdQuery(const char *cpszArgv[])
{
    PWL_IOCTL_PARA para;
    g_pBlk->uCmd = WL_IOCTL_GET_PARAMETER;
    *g_pBlk->pData = 0;
    if (ioctl(g_fd, SIOC_WL_IOCTL, &g_ifreq) == -1) {
        perror("ERROR");
       return FALSE;
    }
    para = (PWL_IOCTL_PARA) g_pBlk->pData;
    printf("DefaultChannel = %d\n",para->DefaultChannel);
    printf("KeepChannel = %d\n",para->Channel);
    if (para->NetworkType == 0){
        printf("NetworkType = Infrastructure [%d]\n",para->NetworkType);
    }
    else{
        printf("NetworkType = Ad-Hoc [%d]\n",para->NetworkType);
    }
    if (para->LLCType == 0){
        printf("LLCType = PCnet-Mobile Compatible[%d]\n",para->LLCType);
    }
    else{
        printf("LLCType = WaveLAN Compatible[%d]\n",para->LLCType);
    }
    switch(para->FrequencyDomain){
        case 0x10:
            printf("FrequencyDomain = FCC\n");
            break;
        case 0x20:
            printf("FrequencyDomain = IC\n");
            break;
        case 0x30:
            printf("FrequencyDomain = ETSI\n");
            break;
    	case 0x31:
            printf("FrequencyDomain = Spain\n");
            break;
    	case 0x32:
            printf("FrequencyDomain = France\n");
            break;
    	case 0x40:
            printf("FrequencyDomain = MKK\n");
            break;
        default:
            printf("FrequencyDomain = Unknow\n");
            break;
    }
    printf("Firmware Version %d.%d.%d\n",para->Version[0],(para->Version[1] >>4) & 0xf,(para->Version[1] & 0xf));    
    para->ESSID[para->ESSID[1]+2]=0;
    printf("ESSID: %s\n",(char *) &(para->ESSID[2])); 
    para->KeepESSID[para->KeepESSID[1]+2]=0;
    printf("KeepESSID: %s\n",(char *) &(para->KeepESSID[2])); 
    return TRUE;
}

BOOL WL_CmdSet(const char *cpszArgv[])
{
    PWL_IOCTL_PARA para;
    WL_IOCTL_PARA pdata;
    g_pBlk->uCmd = WL_IOCTL_GET_PARAMETER;
    *g_pBlk->pData = 0;
    if (ioctl(g_fd, SIOC_WL_IOCTL, &g_ifreq) == -1) {
        perror("ERROR");
       return FALSE;
    }
    para = (PWL_IOCTL_PARA) g_pBlk->pData;
    sscanf(cpszArgv[0], "%d", &pdata.DefaultChannel);
    sscanf(cpszArgv[1], "%d", &pdata.NetworkType);
    sscanf(cpszArgv[2], "%d", &pdata.LLCType);
    if (strcasecmp(cpszArgv[3],"ANY") != 0){
        sscanf(cpszArgv[3], "%s", &pdata.ESSID[2]);
    }
    else{
        sscanf("ANY", "%s", &pdata.ESSID[2]);
    }
    pdata.ESSID[0]=0;
    pdata.ESSID[1]=strlen((char *) &pdata.ESSID[2]);
    printf("Set CHANNEL = %d\n", pdata.DefaultChannel);
    switch(para->FrequencyDomain){
        case 0x10:
            if (pdata.DefaultChannel < 1 || pdata.DefaultChannel > 11) {
                printf("ERROR: CHANNEL must be between 1 and 11\n");
                return FALSE;
            }
            break;
        case 0x20:
            if (pdata.DefaultChannel < 1 || pdata.DefaultChannel > 11) {
                printf("ERROR: CHANNEL must be between 1 and 11\n");
                return FALSE;
            }
            break;
        case 0x30:
            if (pdata.DefaultChannel < 1 || pdata.DefaultChannel > 13) {
                printf("ERROR: CHANNEL must be between 1 and 13\n");
                return FALSE;
            }
            break;
        case 0x31:
            if (pdata.DefaultChannel < 10 || pdata.DefaultChannel > 11) {
                printf("ERROR: CHANNEL must be between 10 and 11\n");
                return FALSE;
            }
            break;
        case 0x32:
            if (pdata.DefaultChannel < 10 || pdata.DefaultChannel > 13) {
                printf("ERROR: CHANNEL must be between 10 and 13\n");
                return FALSE;
            }
            break;
        case 0x40:
            if (pdata.DefaultChannel != 14) {
                printf("ERROR: CHANNEL must be 14\n");
                return FALSE;
            }
            break;
        default:
            if (pdata.DefaultChannel < 1 || pdata.DefaultChannel > 14) {
                printf("ERROR: CHANNEL must be between 1 and 14\n");
                return FALSE;
            }
            break;
    }
    printf("Set NetworkType = %d\n", pdata.NetworkType);
    if ((pdata.NetworkType != 0) && (pdata.NetworkType != 1)){
       printf("ERROR: NetworkType be 0 (Infrastructure) or 1 (Ad-Hoc)\n");
       return FALSE;
    }
    printf("Set LLCType = %d\n", pdata.LLCType);
    if ((pdata.LLCType != 1) && (pdata.LLCType != 2)){
       printf("ERROR: LLCType be 1 (WaveLAN Compatible) or 2 (PCnet-Mobile Compatible)\n");
       return FALSE;
    }
    printf("Set ESSID : %s\n",(char *) &(pdata.ESSID[2]));
    g_pBlk->uCmd = WL_IOCTL_SET_PARAMETER;
    g_pBlk->uLen = sizeof(WL_IOCTL_PARA);
    g_pBlk->pData=(unsigned char *) &pdata;
    if (ioctl(g_fd, SIOC_WL_IOCTL, &g_ifreq) == -1) {
        perror("ERROR");
        return FALSE;
    }

    return TRUE;
}


WL_CMD g_WlCmd[] = {
    { "reset",      1, WL_CmdReset},     /* (write only) / now           */
    { "firmware",   1, WL_CmdFlash},     /* (write only) / now           */
    { "query",      0, WL_CmdQuery },  /* query parameter                 */
    { "set",        3, WL_CmdSet },  /* set parameter                 */
    { NULL, 0, NULL }
};

int FindCommandIndex(const char *cpszInput)
{
    int         i;
    const char *cpszCmd;
    
    for (i = 0; (cpszCmd = g_WlCmd[i].cpszCmd) != NULL; i++) {
       if (strcasecmp(cpszInput, cpszCmd) == 0)
            return i;
    }
    
    return -1;
}

int main(int argc, char *argv[])
{
    int i;
    
    if (getuid() != 0) {
        printf("\nYou must be root to run %s\n\n", argv[0]);
        return 1;
    }
    
    if (argc < 3) {
	printf("WebWorld WL24xx Configuration Tool\n");
        printf("Usage: %s interface cmd argv1 [argv2, ...]\n\n", argv[0]);
        printf("Example: %s twl0 reset now\n",argv[0]);
        printf("         %s twl0 firmware filename\n",argv[0]);
        printf("         %s twl0 query\n",argv[0]);
        printf("         %s twl0 set Channel# NetworkType LLCType ESSID\n",argv[0]);
        return 2;
    }
    
    if ((g_fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("ERROR - socket()");
        return 3;
    }

    strcpy(g_ifreq.ifr_name, argv[1]);
    g_pBlk = (WL_IOCTL_BLK *) &g_ifreq.ifr_data;
    g_pBlk->pData = g_userdata;

    /* program & interface had been processed */
    for (i = 2; i < argc; ) {
        int nNeededArgc;
        BOOL bResult;
        int nCmdIndex = FindCommandIndex(argv[i]);
        
        if (nCmdIndex == -1) {
            printf("Unknow command - %s\n", argv[i]);
            return -1;
        }
        i++; 
        nNeededArgc = g_WlCmd[nCmdIndex].nArgc;
        if (argc - i < nNeededArgc) {
            printf("Insufficient arguments for command '%s' (at least %d)\n",
                    g_WlCmd[nCmdIndex].cpszCmd, nNeededArgc);
            return -1;
        }
            
        bResult = (*g_WlCmd[nCmdIndex].CallFunc)(&argv[i]);
        if (!bResult) {
            printf("Command '%s' failed!!\n\n", 
                    g_WlCmd[nCmdIndex].cpszCmd);
            return -1;
        }
        return 0;
    }
    return 0;
}


